@@ -1,6 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2009, Nokia
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -27,10 +26,7 @@
 
 #include <libtracker-common/tracker-dbus.h>
 #include <libtracker-db/tracker-db-dbus.h>
-#include <libtracker-db/tracker-db-backup.h>
-#include <libtracker-db/tracker-db-journal.h>
 #include <libtracker-db/tracker-db-interface-sqlite.h>
-#include <libtracker-db/tracker-db-journal.h>
 
 #include <libtracker-data/tracker-data-update.h>
 #include <libtracker-data/tracker-data-query.h>
@@ -38,8 +34,7 @@
 
 #include "tracker-store.h"
 
-#define TRACKER_STORE_TRANSACTION_MAX                   4000            /* At commit is journal fsynced too */
-#define TRACKER_STORE_JOURNAL_TIMEOUT_BEFORE_BACKUP     (60 * 60 * 2)   /* Two hours before backup */
+#define TRACKER_STORE_TRANSACTION_MAX                   4000
 
 typedef struct {
 	gboolean  have_handler, have_sync_handler;
@@ -109,6 +104,18 @@ process_turtle_file_part (TrackerTurtleReader *reader, GError **error)
 
 	i = 0;
 
+	/* There is no logical structure in turtle files, so we have no choice
+	 * but fallback to fixed number of statements per transaction to avoid
+	 * blocking tracker-store.
+	 * Real applications should all use SPARQL update instead of turtle
+	 * import to avoid this issue.
+	 */
+	tracker_data_begin_transaction (&new_error);
+	if (new_error) {
+		g_propagate_error (error, new_error);
+		return FALSE;
+	}
+
 	while (new_error == NULL && tracker_turtle_reader_next (reader, &new_error)) {
 		/* insert statement */
 		if (tracker_turtle_reader_get_object_is_uri (reader)) {
@@ -129,13 +136,28 @@ process_turtle_file_part (TrackerTurtleReader *reader, GError **error)
 
 		i++;
 		if (!new_error && i >= 10) {
+			tracker_data_commit_transaction (&new_error);
+			if (new_error) {
+				tracker_data_rollback_transaction ();
+				g_propagate_error (error, new_error);
+				return FALSE;
+			}
 			/* return to main loop */
 			return TRUE;
 		}
 	}
 
 	if (new_error) {
+		tracker_data_rollback_transaction ();
 		g_propagate_error (error, new_error);
+		return FALSE;
+	}
+
+	tracker_data_commit_transaction (&new_error);
+	if (new_error) {
+		tracker_data_rollback_transaction ();
+		g_propagate_error (error, new_error);
+		return FALSE;
 	}
 
 	return FALSE;
@@ -147,7 +169,7 @@ begin_batch (TrackerStorePrivate *private)
 	if (!private->batch_mode) {
 		/* switch to batch mode
 		   delays database commits to improve performance */
-		tracker_data_begin_transaction ();
+		tracker_data_begin_db_transaction ();
 		private->batch_mode = TRUE;
 		private->batch_count = 0;
 	}
@@ -158,35 +180,13 @@ end_batch (TrackerStorePrivate *private)
 {
 	if (private->batch_mode) {
 		/* commit pending batch items */
-		tracker_data_commit_transaction ();
-
-		/* The on_statements_committed in tracker-resources.c performs
-		 * the flush on the journal, I can only register one callback
-		 * for this atm, so that's why it's called over there as a
-		 * tracker_store_flush_journal */
+		tracker_data_commit_db_transaction ();
 
 		private->batch_mode = FALSE;
 		private->batch_count = 0;
 	}
 }
 
-static void
-on_backup_done (GError *error, gpointer user_data)
-{
-	if (!error) {
-		tracker_db_journal_truncate ();
-	}
-}
-
-static void
-log_to_journal (TrackerStorePrivate *private, const gchar *query)
-{
-	tracker_db_journal_log (query);
-
-	if (tracker_db_journal_get_size () > TRACKER_DB_JOURNAL_MAX_SIZE) {
-		tracker_db_backup_save (on_backup_done, NULL, NULL);
-	}
-}
 
 static gboolean
 queue_idle_handler (gpointer user_data)
@@ -204,10 +204,6 @@ queue_idle_handler (gpointer user_data)
 
 		tracker_data_update_sparql (task->data.update.query, &error);
 
-		if (private->start_log) {
-			log_to_journal (private, task->data.update.query);
-		}
-
 		if (!error) {
 			private->batch_count++;
 			if (private->batch_count >= TRACKER_STORE_TRANSACTION_MAX) {
@@ -289,14 +285,6 @@ queue_idle_handler (gpointer user_data)
 	return !g_queue_is_empty (private->queue);
 }
 
-static gboolean
-sync_idle_handler (gpointer user_data)
-{
-	tracker_db_backup_save (on_backup_done, NULL, NULL);
-
-	return TRUE;
-}
-
 static void
 queue_idle_destroy (gpointer user_data)
 {
@@ -306,64 +294,8 @@ queue_idle_destroy (gpointer user_data)
 }
 
 
-static void
-sync_idle_destroy (gpointer user_data)
-{
-	TrackerStorePrivate *private = user_data;
-
-	tracker_db_journal_close ();
-
-	private->have_sync_handler = FALSE;
-}
-
 void
-tracker_store_flush_journal (void)
-{
-	TrackerStorePrivate *private;
-
-	private = g_static_private_get (&private_key);
-	g_return_if_fail (private != NULL);
-
-	tracker_db_journal_fsync ();
-}
-
-static void
-internal_play_journal (TrackerStorePrivate *private)
-{
-	TrackerJournalContents *lines;
-
-	lines = tracker_db_journal_get_contents (TRACKER_STORE_TRANSACTION_MAX);
-
-	if (lines) {
-		guint i;
-
-		tracker_data_begin_transaction ();
-		private->start_log = FALSE;
-		for (i = 0; i < lines->len; i++) {
-			const gchar *line = g_ptr_array_index (lines, i);
-			tracker_store_sparql_update (line, NULL);
-		}
-		tracker_db_journal_truncate ();
-		private->start_log = TRUE;
-		tracker_data_commit_transaction ();
-
-		tracker_db_journal_free_contents (lines);
-	}
-}
-
-void
-tracker_store_play_journal (void)
-{
-	TrackerStorePrivate *private;
-
-	private = g_static_private_get (&private_key);
-	g_return_if_fail (private != NULL);
-
-	internal_play_journal (private);
-}
-
-void
-tracker_store_init (gboolean load_journal)
+tracker_store_init (void)
 {
 	TrackerStorePrivate *private;
 
@@ -375,19 +307,6 @@ tracker_store_init (gboolean load_journal)
 	                      private,
 	                      private_free);
 
-	if (load_journal) {
-		internal_play_journal (private);
-	}
-
-	tracker_db_journal_open ();
-
-	private->start_log = TRUE;
-
-	private->sync_handler = g_timeout_add_seconds_full (G_PRIORITY_LOW,
-	                                                    TRACKER_STORE_JOURNAL_TIMEOUT_BEFORE_BACKUP,
-	                                                    sync_idle_handler,
-	                                                    private,
-	                                                    sync_idle_destroy);
 }
 
 void
@@ -521,18 +440,14 @@ tracker_store_sparql_update (const gchar *sparql,
 
 	if (private->batch_mode) {
 		/* commit pending batch items */
-		tracker_data_commit_transaction ();
+		tracker_data_commit_db_transaction ();
 		private->batch_mode = FALSE;
 		private->batch_count = 0;
 	}
 
-	tracker_data_begin_transaction ();
+	tracker_data_begin_db_transaction ();
 	tracker_data_update_sparql (sparql, error);
-	tracker_data_commit_transaction ();
-
-	if (private->start_log) {
-		log_to_journal (private, sparql);
-	}
+	tracker_data_commit_db_transaction ();
 
 }
 
@@ -550,18 +465,14 @@ tracker_store_sparql_update_blank (const gchar *sparql,
 
 	if (private->batch_mode) {
 		/* commit pending batch items */
-		tracker_data_commit_transaction ();
+		tracker_data_commit_db_transaction ();
 		private->batch_mode = FALSE;
 		private->batch_count = 0;
 	}
 
-	tracker_data_begin_transaction ();
+	tracker_data_begin_db_transaction ();
 	blank_nodes = tracker_data_update_sparql_blank (sparql, error);
-	tracker_data_commit_transaction ();
-
-	if (private->start_log) {
-		log_to_journal (private, sparql);
-	}
+	tracker_data_commit_db_transaction ();
 
 	return blank_nodes;
 }
